import {Component, ComponentInterface, Prop, h} from '@stencil/core';

@Component({
  tag: 'hy-image',
  styleUrl: 'image.scss',
  shadow: false,
})
export class Image implements ComponentInterface {
  @Prop() imageUrl?: string;
  @Prop() imageAlt: string = null;
  @Prop() caption: string = null;
  @Prop() aspectRatioWidth: number = 16;
  @Prop() aspectRatioHeight: number = 9;

  render() {
    const classAttributes = ['hy-image', this.caption ? 'hy-image--with-caption' : null].join(' ');

    const aspect = (this.aspectRatioHeight / this.aspectRatioWidth) * 100;
    const aspectRatio = {
      '--aspectRatio': `${aspect}%` as 'aspectRatio',
    };
    return [
      <div class={classAttributes}>
        <div class="hy-image__image-container" style={aspectRatio}>
          <img alt={this.imageAlt} class="hy-image__image" src={this.imageUrl} />
        </div>
        {this.caption && <div class="hy-image__caption">{this.caption}</div>}
      </div>,
      <hy-box mb="1.75, 1.75, 2, 2.5" />,
    ];
  }
}
