import {Component, h, Prop} from '@stencil/core';
import {css} from 'emotion';

const createCss = (justify: string, align: string, margin: number) => {
  return css`
    display: flex;
    justify-content: ${justify};
    align-items: ${align};
    > * + * {
      margin-left: ${margin}rem;
    }
  `;
};

@Component({
  tag: 'hy-row',
  styleUrl: 'row.css',
  shadow: false
})
export class Row {
  /**
   * justify-content property.
   */
  @Prop() justify: string = 'stretch';
  /**
   * justify-content property.
   */
  @Prop() align: string = 'center';
  /**
   * Left margin to be added to adjacent items.
   */
  @Prop() margin: number = 1;
  render() {
    return (
      <div class={createCss(this.justify, this.align, this.margin)}>
        <slot></slot>
      </div>
    );
  }
}
